#!/usr/bin/perl -w

use strict;
use DBI;
use CGI;
my $dbname = 'tbdb';

main();

###########################
# Input: nodekey cnet vpnif
# Output: none
#	This corrects the database interfaces table for homenet nodes so they're talked to
#	over the OpenVPN control network.
###########################
# Note that the database utility functions are taken from a
# public-domain project (POUND) and are thus not themselves
# candidates for exclusive copyright. Otherwise, this code
# is written by Pat Gunn as part of the Emulab project and is released under
# the same license as the rest of the emulab project.
###################################################

sub main
{
my $query = new CGI;

my $wakey = $query->param('nodekey');
$wakey =~ s/\n\f\r//g;
my $vpnip = $query->param('cnet');
$vpnip =~ s/\n\f\r//g;
my $vpnif = $query->param('vpnif');
$vpnif =~ s/\n\f\r//g;

if(! ( (defined $wakey) && (defined $vpnip) && (defined $vpnif)) )
	{
	do_error($query, "Invalid args [$wakey] [$vpnip] [$vpnif]\n");
	}

my $dbh = db_connect();
my $nodeid = get_nodeid_from_key($dbh, $wakey);
if(! defined($nodeid)) {do_error($query, "Unknown key [$wakey]");}
correct_interfaces_table($dbh, $nodeid, $vpnip, $vpnif);
print STDERR "hnregister served key [$wakey] OK\n";
print $query->header(-type => 'text/plain');
print "OK\n";
}

sub correct_interfaces_table
{
my ($dbh, $nodeid, $vpnip, $vpnif) = @_;
# Steps:
# 1) Remove existing interfaces for the node
# 2) Insert the new control network for the node
# XXX Concerns: I hope none of the later stuff tries to bring interfaces up or down or reprogram them,
#	because that could easily confuse OpenVPN
# 	Would it be safer to forcibly register the public interface too as an interface of type 'other'?
# XXX Also: I hope I can get away with a blank uuid and registering this as the control interface even
#	as it'll be used for both the control plane and a data plane.
my $iftu = $dbh->prepare("DELETE FROM interfaces WHERE node_id=?");
$iftu->execute($nodeid);
my $ifti = $dbh->prepare("INSERT INTO interfaces(node_id, IP, iface, role, uuid) values (?,?,?,?,?)");
$ifti->execute($nodeid, $vpnip, $vpnif, 'ctrl', '');
}

sub db_connect
{
my $dbh = DBI->connect("dbi:mysql:dbname=$dbname");
if($dbh->ping)
        {
#	if($debug) {print "Connected";}
        }
else    
	{die "Not connected: $!\n";}
return $dbh;
}

sub get_nodeid_from_key
{
my ($dbh, $wakey) = @_;

my $nq = $dbh->prepare("SELECT node_id FROM widearea_nodeinfo WHERE privkey=?");
$nq->execute($wakey);
return get_dbval($nq);
}

sub get_dbval($)
{ # Return result from a query that at most generates a single value
my ($dbq) = @_;
my $res = $dbq->fetchall_arrayref();
return $$res[0][0];
}

sub do_error
{
my ($query, $err) = @_;
print $query->header(-type => 'text/plain');
die "ERROR (hnregister)! $err!\n";
}

sub get_dbresults($)
{ # Returns a proper hash instead of a reference to one, for FIRST record returned
my ($dbq) = @_;
my $reshdl = $dbq->fetchrow_hashref();
my %returner;
if(defined($reshdl))
	{
	%returner = %$reshdl;
	}
return %returner;
}

sub slurp_file
{
my ($fts) = @_;
open(FTS, $fts) || die "Failed to open $fts:$!\n";
local $/;
my $ret = readline(FTS);
close(FTS);
return $ret;
}

