#!/usr/bin/perl -w

use strict;
use DBI;
use CGI;
my $dbname = 'tbdb';
my $boss = q{128.2.222.223}; # FIXME Get this from the installed Emulab!
my $cacrt = q{/usr/testbed/etc/openvpn.crt};
my $oconfig = q{/usr/testbed/etc/openvpn-clients.cfg};
my $clientpath = q{/etc/openvpn}; # Where the client should untar this thing..
use Archive::Tar;

main();

###########################
# Input: wanode key
# Output: tarball with all the config stuff for that node
###########################
# Note that the database utility functions are taken from a
# public-domain project (POUND) and are thus not themselves
# candidates for exclusive copyright. Otherwise, this code
# is written by Pat Gunn as part of the Emulab project and is released under
# the same license as the rest of the emulab project.
###################################################

sub main
{
my $query = new CGI;

if(! -f $cacrt) {die "No CA file installed!\n"};
if(! -f $oconfig) {die "No openvpn client config template installed!\n"};


my $wakey = $query->param('key');
$wakey =~ s/\n\f\r//g;

my $dbh = db_connect();
my $nodeid = get_nodeid_from_key($dbh, $wakey);
if(! defined($nodeid)) {do_error($query, "Unknown key [$wakey]");}
my $clientconf = fix_oconfig($oconfig, $nodeid);


my $memtar = Archive::Tar->new();

print $query->header(-type => 'application/x-tar');
my $dq = $dbh->prepare("SELECT connection_id,file,data FROM sw_configfiles WHERE node_id=? AND swid='wa_openvpn' AND connection_id=0");
$dq->execute($nodeid);

while(my %thisrow = get_dbresults($dq))
	{
	$memtar->add_data("$thisrow{connection_id}/$nodeid.$thisrow{file}", $thisrow{data});
	}

$memtar->add_data("ca.crt", slurp_file($cacrt)); # For now, assuming there's only one of these.
$memtar->add_data('openvpn.conf', $clientconf);

my $tardat = $memtar->write(); # give us a string with the in-memory tarball..

print $tardat;
print STDERR "serve_openvpn_keys served key [$wakey] OK\n";
}

sub db_connect
{
my $dbh = DBI->connect("dbi:mysql:dbname=$dbname");
if($dbh->ping)
        {
#	if($debug) {print "Connected";}
        }
else    
	{die "Not connected: $!\n";}
return $dbh;
}

sub get_nodeid_from_key
{
my ($dbh, $wakey) = @_;

my $nq = $dbh->prepare("SELECT node_id FROM widearea_nodeinfo WHERE privkey=?");
$nq->execute($wakey);
return get_dbval($nq);
}

sub get_dbval($)
{ # Return result from a query that at most generates a single value
my ($dbq) = @_;
my $res = $dbq->fetchall_arrayref();
return $$res[0][0];
}

sub do_error
{
my ($query, $err) = @_;
print $query->header(-type => 'text/plain');
print STDERR "Failure in serve_openvpn_keys: $err\n";
die "ERROR (serve_openvpn_keys)! $err!\n";
}

sub get_dbresults($)
{ # Returns a proper hash instead of a reference to one, for FIRST record returned
my ($dbq) = @_;
my $reshdl = $dbq->fetchrow_hashref();
my %returner;
if(defined($reshdl))
	{
	%returner = %$reshdl;
	}
return %returner;
}

sub slurp_file
{
my ($fts) = @_;
open(FTS, $fts) || die "Failed to open $fts:$!\n";
local $/;
my $ret = readline(FTS);
close(FTS);
return $ret;
}

sub fix_oconfig
{
my ($ctemplate, $nodeid) = @_;
my $tdat = slurp_file($ctemplate);
$tdat =~ s/\$boss/$boss/g; # Replace in-file placeholder with our variable
$tdat =~ s/\$cfgpath/$clientpath/g;
$tdat =~ s/\$cfgnwk/0/g; # TODO Figure out how to do this when we're configuring
			# multiple networks..
$tdat =~ s/\$clientname/$nodeid/g;
return $tdat;
}

